/*
	Copyright (c) 2013, Raymond Dodge
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:
		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.
		* Redistributions in binary form must reproduce the above copyright
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.
		* Neither the name "<PRODUCT NAME>" nor the names of its contributors
		  may be used to endorse or promote products derived from this software
		  without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*****************************************************************************
 * A main method that uses 'things for printing a stem-and-leaf plot' to
 * print a stem-and-leaf plot using collected data.
 * @author Raymond Dodge
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "stemAndLeaf.h"

/**
 * A function for use in psort to sort `slData_t`s
 */
int sort_slData(const void* const x, const void* const y) {
	const slData_t* const x2 = (slData_t*) x;
	const slData_t* const y2 = (slData_t*) y;
	
	return (x2->power != y2->power ? x2->power - y2->power :
		(x2->leastSigDig1 != y2->leastSigDig1 ? x2->leastSigDig1 - y2->leastSigDig1 :
		(x2->leastSigDig2 != y2->leastSigDig2 ? x2->leastSigDig2 - y2->leastSigDig2 :
		(x2->leastSigDig3 != y2->leastSigDig3 ? x2->leastSigDig3 - y2->leastSigDig3 :
	0 ))));
}



int main(const int argc, const char** argv) {
	
	if (argc == 1) {
		printf("Stem And Leaf Plotter\n");
		printf("Copyright Raymond Dodge 2013\n");
		printf("\n");
		printf("Syntax:\n");
		printf("\t%s [options] list of numbers\n", argv[0]);
		printf("\t%s [options] @file\n", argv[0]);
		printf("\n");
		printf("Options:\n");
		printf("\t--split     split each major digit into two stems\n");
		
		return 1;
	} else {
		int i;
		int size = 0;
		int maxSize = argc;
		slData_t* values = malloc( sizeof(slData_t) * maxSize );
		graphTypes_t graphType = DECIMAL;
		
		for (i = 1; i < argc; i++) {
			if (strcmp("--split", argv[i]) == 0) {
				graphType = SPLIT_DECIMAL;
			} else if ('@' == argv[i][0]) {
				double nextValue;
				int proceed;
				FILE* file = fopen(argv[i] + 1, "r");
				
				if (!file) {
					fprintf(stderr, "couldn't open %s\n", argv[i] + 1);
				} else {
					proceed = fscanf(file, "%lf", &nextValue);
					
					while (proceed > 0) {
						if (maxSize <= size) {
							maxSize = maxSize * 2;
							values = realloc(values, sizeof(slData_t) * maxSize);
						}
						values[size] = convertDecimalToSlData( nextValue );
						size++;
						
						proceed = fscanf(file, "%lf", &nextValue);
					}
				}
				
				fclose(file);
			} else {
				if (maxSize <= size) {
					maxSize = maxSize * 2;
					values = realloc(values, sizeof(slData_t) * maxSize);
				}
				values[size] = convertStringToSlData(argv[i]);
				size++;
			}
		}
		qsort(values, size, sizeof(slData_t), sort_slData);
		
		printStemAndLeafPlot(size, values, graphType);
		
		free(values);
		return 0;
	}
}
