/*
	Copyright (c) 2013, Raymond Dodge
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:
		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.
		* Redistributions in binary form must reproduce the above copyright
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.
		* Neither the name "<PRODUCT NAME>" nor the names of its contributors
		  may be used to endorse or promote products derived from this software
		  without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*****************************************************************************
 * Methods relating to the printing of a stem-and-leaf plot
 * @author Raymond Dodge
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/**
 * A structure for storing information required to sort and display
 * stems and leaves
 */
typedef struct {
	char power;
	char leastSigDig1;
	char leastSigDig2;
	char leastSigDig3;
} slData_t;

/**
 * Possible methods to display a stemAndLeaf plot
 */
typedef enum {
    /** Each stem is a decimal digit */
	DECIMAL,
    /** Two stems per decimal digit, one for 0-4 leaves
     *  and one for 5-9 leaves
     */
	SPLIT_DECIMAL,
	/** No specified or forced format */
	UNDEFINED
} graphTypes_t;

/**
 * Converts a number-encoded-as-a-cstring into a slData_t
 */
slData_t convertStringToSlData(const char* const str);

/**
 * Converts a number into a slData_t
 */
slData_t convertDecimalToSlData(const double value);

/**
 * Print a stem-and-leaf plot using printf.
 * @param valuec the number of elements in valuea
 * @param valuea numbers to print in the stem and leaf plot
 * @param graphType hits on how to print the graph
 * @pre valuea is sorted.
 */
void printStemAndLeafPlot(
		const int valuec,
		const slData_t* const valuea,
		const graphTypes_t graphType
);

